package kr.nipa.smartui.phone.framework.common.cough;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ClosedChannelException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.Iterator;
import java.util.Set;

import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class MessageReceiver extends Thread {
	private static final int BUFFER_SIZE = 255;
	
	private ByteBuffer _ReadBuffer;
	private ByteBuffer _WriteBuffer;	
	private CharsetDecoder _Decoder;	
	
	private static final long CHANNEL_WRITE_SLEEP = 10L;
	
	private SocketChannel _SocketChannel;
	
	private Selector _ReadSelector;	
	
//	private boolean _IsConnected;
	private boolean _IsRunning;
	
	private Handler _Handler = null;
	
	public MessageReceiver(SocketChannel socketChannel, Selector readSelector, Handler handler) {
		_ReadBuffer = ByteBuffer.allocateDirect(BUFFER_SIZE);
		_WriteBuffer = ByteBuffer.allocateDirect(BUFFER_SIZE);		
		_Decoder = Charset.forName("US-ASCII").newDecoder();
		
		_SocketChannel = socketChannel;		
		_ReadSelector = readSelector;		
//		_IsConnected = true;
		
		_IsRunning = false;
		
		_Handler = handler;	
	}
	
	@Override
	public void run() {		
		_IsRunning = true;
		
		while (_IsRunning) {
			readIncomingMessages();
			
			try {Thread.sleep(50);} catch (InterruptedException ie) {ie.printStackTrace();}
		}
	}
	
	public void shutdown(SocketChannel socketChannel) throws IOException {
		socketChannel.close();
//		_IsConnected = false;
		
		_IsRunning = false;		
		interrupt();
	}
	
	private void readIncomingMessages() {
		Message msg = null;
		
		try {
			_ReadSelector.selectNow();
			
			Set readyKeys = _ReadSelector.selectedKeys();
			
			Iterator i = readyKeys.iterator();
			while (i.hasNext()) {
				SelectionKey key = (SelectionKey)i.next();
				i.remove();
				
				SocketChannel channel = (SocketChannel)key.channel();
				
				_ReadBuffer.clear();				
				long nBytes = channel.read(_ReadBuffer);
				
				if (nBytes == -1) {
//					_IsConnected = false;
					Log.d("Smart UI", "disconnected form server: end-of-stream");
					
//					channel.close();
					shutdown(channel);
				} 
				else {					
					StringBuffer sbCommands = (StringBuffer)key.attachment();
					
					_ReadBuffer.flip();
					
					String commands = _Decoder.decode(_ReadBuffer).toString(); sbCommands.append(commands);
					Log.d("Smart UI", sbCommands.toString());
					String[] commandList = sbCommands.toString().split("\\|"); sbCommands.setLength(0);
					
					_ReadBuffer.clear();					
					
					if (commandList.length >= 1) {
						if (commandList[0].equals("2")) {
							if (commandList[1].equals("11")) {
								//
							}
							else if (commandList[1].equals("12")) {
								//
							}
							else if (commandList[1].equals("21")) {
								if (!commandList[2].equals("p") && !commandList[2].equals("d")) return;
								
								msg = Message.obtain(_Handler, 1, 21, 0, commands);			
								_Handler.sendMessage(msg);
							}
							else if (commandList[1].equals("22")) {
								if (!commandList[2].equals("p") && !commandList[2].equals("d")) return;
								
								msg = Message.obtain(_Handler, 1, 22, 0, commands);			
								_Handler.sendMessage(msg);
							}
						}
					}
				}
			}
		} catch (IOException ioe) {
			Log.e("Smart UI", ioe.getStackTrace().toString() + "\n" + ioe.getMessage().toString());
		} catch (Exception e) {
			Log.e("Smart UI", e.getStackTrace().toString() + "\n" + e.getMessage().toString());
		}
	}
	
	public void sendMessage(String msg) {
		prepareWriteBuffer(msg);
		
		write(_SocketChannel, _WriteBuffer);
	}
	
	private void prepareWriteBuffer(String msg) {
		_WriteBuffer.clear();
		
		_WriteBuffer.put(msg.getBytes());
//		_WriteBuffer.putChar('\n');
		_WriteBuffer.flip();
	}
	
	private void write(SocketChannel channel, ByteBuffer writeBuffer) {
		long nBytes = 0;
		
		long toWrite = writeBuffer.remaining();
		
		try {
			while (nBytes != toWrite) {
				nBytes += channel.write(writeBuffer);
				
				try {Thread.sleep(CHANNEL_WRITE_SLEEP);} catch (InterruptedException e) {Log.e("Smart UI", e.getStackTrace().toString() + "\n" + e.getMessage().toString());}
			}
		} 
		catch (ClosedChannelException cce) {
			Log.e("Smart UI", cce.getStackTrace().toString() + "\n" + cce.getMessage().toString());
		} 
		catch (Exception e) {
			Log.e("Smart UI", e.getStackTrace().toString() + "\n" + e.getMessage().toString());
		}
		
		writeBuffer.rewind();
	}
	
//	public boolean isConnected() {
//		return _IsConnected;
//	}
	
	public boolean isRunning() {
		return _IsRunning;
	}
}
