package kr.nipa.smartui.phone.framework.common.cough;

import java.io.IOException;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;

import kr.nipa.smartui.phone.R;
import kr.nipa.smartui.phone.framework.db.cough.DbOpenHelper;
import kr.nipa.smartui.phone.framework.util.IoUtil;
import kr.nipa.smartui.phone.framework.util.PreferenceUtil;
import kr.nipa.smartui.phone.ui.cough.message.coms0200a;
import android.app.DownloadManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class MessageReceiverController extends Thread {
	private Context _Context;
	private DbOpenHelper _DbOpenHelper;
	
	private String _HostIpAddr;
	private int _HostPortNum;
	
	private SocketChannel _SocketChannel;	
	private Selector _ReadSelector;		
	private MessageReceiver _MessageReceiver;
	
	private boolean _IsConnected;
	
	private NotificationManager _NotificationManager;
	private DownloadManager _DownloadManager;
	
	private boolean _IsRunning;
	
	public MessageReceiverController(Context context) {
		_Context = context;		
		_DbOpenHelper = new DbOpenHelper(_Context);
		
		_HostIpAddr = "210.115.36.206";
		_HostPortNum = 10010;
		
		_SocketChannel = null;
		_ReadSelector = null;
		_MessageReceiver = null;
		
		_IsConnected = false;
		 
		_NotificationManager = (NotificationManager)context.getSystemService(Context.NOTIFICATION_SERVICE);
		_DownloadManager = (DownloadManager)context.getSystemService(Context.DOWNLOAD_SERVICE);
		
		_IsRunning = false;
	}
	
	@Override
	public void run() {
		Intent intent = null;
		
		_IsRunning = true;
		
		while (_IsRunning) {
			if (_MessageReceiver == null) {
				_IsConnected = connect();
				if (!_IsConnected) continue;
//				Log.d("Smart UI", "_MessageReceiver == null, Connected");
				_MessageReceiver = new MessageReceiver(_SocketChannel, _ReadSelector, _ReceivedDataHandler);
		     	_MessageReceiver.setDaemon(true);
		     	_MessageReceiver.start();
		     	
		     	try {Thread.sleep(300);} catch (InterruptedException e) {e.printStackTrace();}
		     	
		     	// 알림
				intent = new Intent();
				intent.setAction("kr.nipa.smartui.pad.intent.action.CONNECTED_MESSAGESERVER");
				_Context.sendBroadcast(intent);
			}
			else {
				if (!_MessageReceiver.isRunning()) {
//					Log.d("Smart UI", "_MessageReceiver == Not Running");
					_MessageReceiver = null;					
					_IsConnected = false;
					
					_IsConnected = connect();
					if (!_IsConnected) continue;
//					Log.d("Smart UI", "_MessageReceiver == Not Running, Connected");
					_MessageReceiver = new MessageReceiver(_SocketChannel, _ReadSelector, _ReceivedDataHandler);
			     	_MessageReceiver.setDaemon(true);
			     	_MessageReceiver.start();
			     	
			     	try {Thread.sleep(300);} catch (InterruptedException e) {e.printStackTrace();}
			     	
			     	// 알림
					intent = new Intent();
					intent.setAction("kr.nipa.smartui.pad.intent.action.CONNECTED_MESSAGESERVER");
					_Context.sendBroadcast(intent);
				}
			}
		}
	}
	
	public void shutdown() throws IOException {
		_MessageReceiver.shutdown(_SocketChannel); _MessageReceiver = null;
		_IsConnected = false;
		
		_IsRunning = false;		
		interrupt();
	}
	
	public boolean connect() {
		boolean ret = false;
		
		try {
			InetAddress addr = InetAddress.getByName(_HostIpAddr);
			_SocketChannel = SocketChannel.open(new InetSocketAddress(addr, _HostPortNum));			
			_SocketChannel.configureBlocking(false);
			
			_ReadSelector = Selector.open();
			_SocketChannel.register(_ReadSelector, SelectionKey.OP_READ, new StringBuffer());
			
//			Log.d("Smart UI", "connected to server ...");
			
			ret = true;
		} 
		catch (UnknownHostException uhe) {
			Log.e("Smart UI", uhe.getStackTrace().toString() + "\n" + uhe.getMessage().toString());
			
			ret = false;			
		} 
		catch (ConnectException ce) {
			Log.e("Smart UI", ce.getStackTrace().toString() + "\n" + ce.getMessage().toString());
			
			ret = false;
		} 
		catch (Exception e) {
			Log.e("Smart UI", e.getStackTrace().toString() + "\n" + e.getMessage().toString());
			
			ret = false;
		}
		
		return ret;
	}
	
	private Handler _ReceivedDataHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			Notification noti = null;
			PendingIntent content = null;
			
			if (msg.what == 1) {
				if (msg.arg1 == 21) {
					String commands = String.valueOf(msg.obj);
					String[] commandList = commands.toString().split("\\|");
					
					// 수신 메시지가 있음을 알린다.
					noti = new Notification(android.R.drawable.stat_notify_chat, commandList[6], System.currentTimeMillis());
					noti.defaults |= Notification.DEFAULT_SOUND;
					noti.flags |= Notification.FLAG_SHOW_LIGHTS;
					
					Intent intent = new Intent(_Context, coms0200a.class);
					intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
					
					content = PendingIntent.getActivity(_Context, 0, intent, 0);					
					noti.setLatestEventInfo(_Context, "From. " + commandList[5] + "[" + commandList[4] + "] ", commandList[6], content);
					
					_NotificationManager.notify(1, noti);
					
					// 수신 메시지를 등록한다.
					registerMessage("1", "", commandList[6], PreferenceUtil.getInstance().getDoctorId(_Context), commandList[4], commandList[5]);
				}
				else if (msg.arg1 == 22) {
					String commands = String.valueOf(msg.obj);
					String[] commandList = commands.toString().split("\\|");
					
					String name = commandList[6].substring(commandList[6].lastIndexOf("/") + 1, commandList[6].length());
					String contents = "This is a voice message.";
					
					String notiTitle = "From. " + commandList[5] + "[" + commandList[4] + "] " + contents;
					
					if (!IoUtil.getInstance(_Context).ExternalStorageIsMounted()) {
						Log.d("Smart UI", _Context.getString(R.string.message_Your_machines_external_storage_is_unmounted));
						
						return;
					}
					
					Uri sourceUri = Uri.parse(commandList[6]);
					DownloadManager.Request req = new DownloadManager.Request(sourceUri);
					req.setTitle(notiTitle);
					req.setDescription(notiTitle);
					req.setAllowedNetworkTypes(DownloadManager.Request.NETWORK_WIFI);
					req.setDestinationInExternalPublicDir(Environment.DIRECTORY_DOWNLOADS, name);
					Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).mkdirs();
		 			
					long downloadId = _DownloadManager.enqueue(req);
					
					// 수신 메시지를 등록한다.
					registerMessage("2", name, contents, PreferenceUtil.getInstance().getDoctorId(_Context), commandList[4], commandList[5]);
				}
			}
		}
	};
	
	public boolean SendLogInReq(String type, String classCode, String id, String name) {
		if (!_IsConnected) return false;
		
		// 로그인 관련 요청메시지를 보낸다.
     	String message = "1" + "|" + 
     					 "11" + "|" + 
     					 type + "|" + 
                         classCode + "|" + 
                         id + "|" + 
                         name;
     	
     	_MessageReceiver.sendMessage(message);
     	
     	return true;
	}
	
	public boolean SendLogOutReq(String type, String classCode, String id) {
		if (!_IsConnected) return false;
		
		// 로그아웃 관련 요청메시지를 보낸다.
     	String message = "1" + "|" + 
     					 "12" + "|" + 
     					 type + "|" + 
                         classCode + "|" + 
                         id;
     	
     	_MessageReceiver.sendMessage(message);
     	
     	return true;
	}
	
	public boolean SendTextMessageTransformReq(String receiverType, String receiverClassCode, String receiverId, String contents) {
		if (!_IsConnected) return false;
		
		// 텍스트메시지 전송관련 요청메시지를 보낸다.
     	String message = "1" + "|" + 
     					 "21" + "|" +
     					 receiverType + "|" +
     					 receiverClassCode + "|" + 
     					 receiverId + "|" + 
                         contents;
     	
     	_MessageReceiver.sendMessage(message);
     	
     	return true;
	}
	
	public boolean SendVoiceMessageTransformReq(String receiverType, String receiverClassCode, String receiverId, String uri) {
		if (!_IsConnected) return false;
		
		// 음성메시지 전송관련 요청메시지를 보낸다.
     	String message = "1" + "|" + 
     					 "22" + "|" + 
     					 receiverType + "|" +
    					 receiverClassCode + "|" + 
    					 receiverId + "|" + 
                         uri;
     	
     	_MessageReceiver.sendMessage(message);
     	
     	return true;
	}
	
	public boolean isConnected() {
		return _IsConnected;
	}
	
	public boolean isRunning() {
		return _IsRunning;
	}
	
	/**
	 * 수신된 메시지를 등록한다.
	 */
    private void registerMessage(String typeCode, String name, String contents, String receiverId, String senderId, String senderName) {
    	SQLiteDatabase db = null;
		
		try {
			// 데이터를 등록한다.
			String sql = "INSERT INTO TbReceivedMessage (TypeCd," 						+
							                    "Name," 					+
							                    "Contents," 					+
							                    "ReceivedDTime," 				+
							                    "ReceiverId," 					+
							                    "SenderId," 					+
							                    "SenderNm) " 					+
		                    	        "VALUES ('" + typeCode + "'," 			+
						                        "'" + name + "'," 			+
						                        "'" + contents + "'," 			+
						                        "DATETIME('now')," 				+
						                        receiverId + "," 				+
						                        "'" + senderId + "'," 			+
						                        "'" + senderName + "')";
			
			db = _DbOpenHelper.getWritableDatabase();			
			db.execSQL(sql);
		}
		finally {
			if (db != null) db.close(); db = null;
		}
    }
}
