package kr.nipa.smartui.phone.framework.common.cough;

import java.text.SimpleDateFormat;
import org.afree.chart.ChartFactory;
import org.afree.chart.AFreeChart;
import org.afree.chart.axis.DateAxis;
import org.afree.chart.plot.XYPlot;
import org.afree.chart.renderer.xy.XYItemRenderer;
import org.afree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.afree.data.xy.XYDataset;
import org.afree.graphics.SolidColor;
import org.afree.ui.RectangleInsets;

import kr.nipa.smartui.phone.framework.base.cough.MonitorChartViewBase;
import android.content.Context;
import android.graphics.Color;

public class MonitorChartView extends MonitorChartViewBase {
//	private static TimeSeries _TimeSeries = null;
//	private static TimeSeriesCollection _TimeSeriesCollection = null;
	
	public MonitorChartView(Context context) {
        super(context);

        final AFreeChart chart = createChart();
        setChart(chart);
    }

    private static AFreeChart createChart() {
//    	_TimeSeries = new TimeSeries("Time");    	
//    	_TimeSeriesCollection = new TimeSeriesCollection(_TimeSeries);
    	
    	AFreeChart chart = ChartFactory.createTimeSeriesChart("",  						// title
												              "Time",             		// x-axis label
												              "Pressure",   			// y-axis label
												              null,						// data
												              false,               		// create legend?
												              false,               		// generate tooltips?
												              false               		// generate URLs?
												              );
    	
    	chart.setBackgroundPaintType(new SolidColor(Color.BLACK));
        chart.setBorderVisible(false);

        XYPlot plot = chart.getXYPlot();
        plot.setAxisOffset(new RectangleInsets(0.0, 0.0, 0.0, 0.0));
        plot.setBackgroundPaintType(new SolidColor(Color.BLACK));
        plot.setNoDataMessage("No data available.");
        plot.setOutlineVisible(false);
        
        plot.getDomainAxis().setAxisLinePaintType(new SolidColor(Color.WHITE));
        plot.getDomainAxis().setTickLabelPaintType(new SolidColor(Color.WHITE));        
        plot.setDomainCrosshairVisible(true);
        plot.setDomainGridlinesVisible(false);
        plot.setDomainZeroBaselinePaintType(new SolidColor(Color.WHITE));
        plot.setDomainZeroBaselineVisible(true);
        
        plot.getRangeAxis().setAxisLinePaintType(new SolidColor(Color.WHITE));
        plot.getRangeAxis().setTickLabelPaintType(new SolidColor(Color.WHITE));
        plot.setRangeCrosshairVisible(true);
        plot.setRangeGridlinesVisible(false);
        plot.setRangeZeroBaselinePaintType(new SolidColor(Color.WHITE));
        plot.setRangeZeroBaselineVisible(true); 

        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer)r;
            
            renderer.setBaseShapesFilled(true);
            renderer.setBaseShapesVisible(false);
            renderer.setDrawSeriesLineAsPath(true);            
            
            renderer.setSeriesStroke(0, 2.5f);
        }

        DateAxis domainAxis = (DateAxis)plot.getDomainAxis();
//        domainAxis.setAutoRange(true);
//        domainAxis.setFixedAutoRange(3000.0);
        domainAxis.setDateFormatOverride(new SimpleDateFormat("s.S"));

//        NumberAxis numberAxis = (NumberAxis)plot.getRangeAxis();
//        numberAxis.setAutoRange(true);
        
        return chart;
    }
    
//    public TimeSeries getTimeSeries() {
//    	return _TimeSeries;
//    }
//    
//    public TimeSeriesCollection getTimeSeriesCollection() {
//    	return _TimeSeriesCollection;
//    }
    
    public XYDataset getData() {
    	AFreeChart chart = getChart();
    	
    	XYPlot plot = (XYPlot)chart.getPlot();    	
    	return plot.getDataset();
    }
    
    public void setData(XYDataset data) {
    	AFreeChart chart = getChart();
    	
    	XYPlot plot = (XYPlot)chart.getPlot();    	
    	plot.setDataset(data);
    	
    	postInvalidate();
    }
}
