package kr.nipa.smartui.phone.framework.db.cough;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

public class DbOpenHelper extends SQLiteOpenHelper {
	public DbOpenHelper(Context context) {
        super(context, "kr.nipa.smartui.phone.db", null, 1);
    }
	
	@Override
	public void onCreate(SQLiteDatabase db) {
		// 공통코드 분류
		db.execSQL("CREATE TABLE TbCommonCodeCateg(Code      		nchar(2)      	  	NOT NULL,"	+
			  								      "Name      		nvarchar(40)     	NOT NULL," 	+
			  								      "CONSTRAINT PkCommonCodeCateg PRIMARY KEY (Code));");

		// 공통코드
		db.execSQL("CREATE TABLE TbCommonCode(CategCd   	nchar(2)      		NOT NULL,"	+
											 "Code      	nchar(2)     		NOT NULL," 	+
											 "Name      	nvarchar(40)    	NOT NULL," 	+
											 "CONSTRAINT PkCommonCodeCateg PRIMARY KEY (CategCd, Code));");
		
		// 의료 기관
		db.execSQL("CREATE TABLE TbHospital(Code      	nvarchar(8)      	NOT NULL," 			+
										   "Name      	nvarchar(40)     	NOT NULL," 			+
										   "Remark    	nvarchar(100)    	NULL," 				+
										   "UseYn     	nchar(1)         	NOT NULL," 			+
										   "CONSTRAINT PkHospital PRIMARY KEY (Code));");		   
		
		// 환자
		db.execSQL("CREATE TABLE TbPatient (No                		integer         	PRIMARY KEY AUTOINCREMENT, " 	+
										   "Id                		nvarchar(10)    	NOT NULL, " 					+
										   "Name              		nvarchar(20)    	NOT NULL, " 					+
										   "Age               		tinyint         	NULL, " 						+
										   "Gender            		nvarchar(2)     	NOT NULL, " 					+
										   "Height            		tinyint         	NULL, " 						+
										   "Weigh             		tinyint         	NULL, " 						+
										   "Diagnosis         		nvarchar(50)    	NULL, " 						+
										   "HospitalCd       		nvarchar(8)     	NOT NULL, " 					+
										   "regDTime          		datetime        	NOT NULL);");
		
		db.execSQL("CREATE UNIQUE INDEX UkTbPatient ON TbPatient (HospitalCd, Id);");
		
		// 모니터 데이터
		db.execSQL("CREATE TABLE TbMonitoredData(No                		integer          	PRIMARY KEY AUTOINCREMENT,"				+
												"Name              		nvarchar(100)     	NOT NULL,"								+
												"InhalePressure    		tinyint          	NOT NULL,"								+
												"ExhalePressure    		tinyint          	NOT NULL,"								+
												"InhaleTime        		float          		NOT NULL,"								+
												"ExhaleTime        		float          		NOT NULL,"								+
												"PauseTime         		float          		NOT NULL,"								+
												"RunCycle          		tinyint          	NOT NULL,"								+
												"IERate           		nvarchar(10)        NOT NULL,"								+
												"RunTime          		float          		NOT NULL,"								+
												"HighLowGb         		nchar(2)         	NOT NULL,"								+
												"RunStartDTime     		datetime         	NOT NULL,"								+
												"RunEndDTime       		datetime         	NOT NULL,"								+
												"ShortDesc         		nvarchar(100)    	NULL,"									+
												"File               	BLOB    			NULL,"									+
												"PatientNo         		integer          	NULL,"									+
												"GatheredDataYn         bool          		NOT NULL,"								+
												"RegDTime          		datetime         	NOT NULL);");
		
		db.execSQL("CREATE UNIQUE INDEX UkTbMonitoredData ON TbMonitoredData (Name);");
		
		// 수신 메시지
		db.execSQL("CREATE TABLE TbReceivedMessage(No               	integer           	PRIMARY KEY AUTOINCREMENT,"				+
												  "TypeCd           	nchar(1)          	NOT NULL,"								+
												  "Name              	nvarchar(100)     	NULL,"									+
												  "Contents         	nvarchar(2000)    	NULL,"									+
											      "ReceivedDTime    	datetime          	NULL,"									+
											      "ReceiverId           nvarchar(10)        NOT NULL,"								+
											      "SenderId            	nvarchar(10)      	NULL,"									+
											      "SenderNm          	nvarchar(20)      	NULL);");
		
//		// 수신 음성메시지 파일
//		db.execSQL("CREATE TABLE TbReceivedVoiceMessageFile(ReceivedMessageNo    	integer       	NOT NULL,"		+
//													       "File                 	binary(10)    	NULL,"			+
//													       "CONSTRAINT PkReceivedVoiceMessageFile PRIMARY KEY (ReceivedMessageNo));");
		
		// 공통코드 관련 초기 데이터를 등록한다.
		db.execSQL("INSERT INTO TbCommonCodeCateg (Code, Name) "	+
				   						  "VALUES ('01', 'Sex');");
		
		db.execSQL("INSERT INTO TbCommonCode (CategCd, Code, Name) "	+
				   					 "VALUES ('01', '01', 'Male');");
		db.execSQL("INSERT INTO TbCommonCode (CategCd, Code, Name) "	+
				   					 "VALUES ('01', '02', 'Female');");	
		
		// 의료기관 관련 초기 데이터를 등록한다.
		db.execSQL("INSERT INTO TbHospital (Code, Name, Remark, UseYn) "	+
				   		           "VALUES ('50000001', 'HALLYM UNIV. MEDICAL CENTER ', null, 1);");
    }
	
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		db.execSQL("DROP TABLE IF EXISTS TbCommonCodeCateg;");
		db.execSQL("DROP TABLE IF EXISTS TbCommonCode;");
		db.execSQL("DROP TABLE IF EXISTS TbHospital;");
    	db.execSQL("DROP TABLE IF EXISTS TbPatient;");
    	db.execSQL("DROP TABLE IF EXISTS TbPrescription;");
    	db.execSQL("DROP TABLE IF EXISTS TbPreSet;");
    	db.execSQL("DROP TABLE IF EXISTS TbUserSet;");
    	db.execSQL("DROP TABLE IF EXISTS TbMonitoredData;");
//    	db.execSQL("DROP TABLE IF EXISTS TbMonitoredDataFile;");
    	db.execSQL("DROP TABLE IF EXISTS TbReceivedMessage;");
    	db.execSQL("DROP TABLE IF EXISTS TbReceivedVoiceMessageFile;");
    	db.execSQL("DROP TABLE IF EXISTS TbReceivedPrescription;");
    	
    	onCreate(db);
    }
}