/*
 * Copyright (C) 2007 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package kr.nipa.smartui.phone.ui;

import android.view.animation.Animation;
import android.view.animation.Transformation;
import android.graphics.Camera;
import android.graphics.Matrix;

public class Animation3DRotator extends Animation {
	private final float _FromDegrees;
	private final float _ToDegrees;
	private final float _CenterX;
	private final float _CenterY;
	private final float _DepthZ;
	private final boolean _Reverse;
	
	private Camera _Camera;

	public Animation3DRotator(float fromDegrees, float toDegrees, float centerX, float centerY, float depthZ, boolean reverse) {
		_FromDegrees = fromDegrees;
		_ToDegrees = toDegrees;
		_CenterX = centerX;
		_CenterY = centerY;
		_DepthZ = depthZ;
		_Reverse = reverse;
	}

	@Override
	public void initialize(int width, int height, int parentWidth, int parentHeight) {
		super.initialize(width, height, parentWidth, parentHeight);
		
		_Camera = new Camera();
	}

	@Override
	protected void applyTransformation(float interpolatedTime, Transformation t) {
		final float fromDegrees = _FromDegrees;
		
		float degrees = fromDegrees + ((_ToDegrees - fromDegrees) * interpolatedTime);
		final float centerX = _CenterX;
		final float centerY = _CenterY;
		final Camera camera = _Camera;

		final Matrix matrix = t.getMatrix();

		camera.save();
		
		if (_Reverse) {
			camera.translate(0.0f, 0.0f, _DepthZ * interpolatedTime);
		} else {
			camera.translate(0.0f, 0.0f, _DepthZ * (1.0f - interpolatedTime));
		}
		
		camera.rotateY(degrees);
		camera.getMatrix(matrix);
		camera.restore();

		matrix.preTranslate(-centerX, -centerY);
		matrix.postTranslate(centerX, centerY);

	}
}
